#!/usr/bin/env python
""" Module to create connection with weaviate and data import scripts """

from .swagger_client import ApiClient


class ToWeaviate():
    """
    This class serves as a intermediate step between data import scripts and doing queries to the Weaviate instance.
    The methods define correct formats for doing queries.
    """

    def __init__(self, config):
        """
            :param config: weaviate configuration dictionairy with token, location and location url
        """   
        self.weaviate_config = config
        self.weaviate_client = ApiClient(
            self.weaviate_config["token"],
            self.weaviate_config["location"])
        self.location_url = self.weaviate_config["urlport"]


    def get_cref_schema(self, uuid):
        """
        Return the schema for property when value is cross reference
            :param uuid: uuid of the thing
        """   
        cref_schema = {}
        cref_schema["locationUrl"] = self.location_url
        cref_schema["type"] = "Thing"
        cref_schema["$cref"] = uuid
        return cref_schema


    def get_uuid(self, class_name, property_name, property_value):
        """
        Return the uuid of a thing based on class and one property
            :param class_name: Class name
            :param property_name: Property name
            :param property_value: Property value
        """   
        body = {
            "query": "{listThings(first:1 schema:\"" +
                     property_name +
                     ":" +
                     str(property_value) +
                     "\", class:\"" +
                     class_name +
                     "\") {things {uuid}}}"}
        response = self.weaviate_client.weaviate_graphql_post(body)
        things = response["data"]["listThings"]["things"]
        # if thing exists: return uuid
        if things is not None and things:
            uuid = things[0]["uuid"]
            return uuid
        # if thing doesn't exists: return None
        return None

    def post_thing(self, class_name, schema):
        """
        Post a thing to Weaviate instance given its schema and class
            :param class_name: Class
            :param schema: Thing schema
        """   
        # how to check if thing already exists?

        body = {}
        body["@context"] = "http://dbpedia.org"
        body["@class"] = class_name
        body["schema"] = schema

        # post thing to weaviate and get a result when posted succesfully
        response = self.weaviate_client.weaviate_things_create(body)

        # return uuid if successfully posted thing
        uuid = response["thingId"]
        return uuid

    def update_thing(self, uuid, operation, path, value):
        """
        Update thing by json patch
            :param uuid: uuid of the Thing
            :param operation: Update operation. E.g. "replace"
            :param path: Path
            :param value: New value
        """   
        body = {}
        body["op"] = operation
        body["path"] = path
        body["value"] = value

        # patch thing to weaviate and get a result when patch was succesful
        self.weaviate_client.weaviate_things_patch(uuid, [body])
        return

    def update_thing_schema(self, uuid, schema):
        """
        Patch the schema of a Thing
            :param uuid: uuid of the Thing
            :param schema: new schema of the thing
        """   
        body = {}
        body["op"] = "replace"
        body["path"] = "/schema"
        body["value"] = schema

        # patch thing to weaviate and get a result when patch was succesful
        self.weaviate_client.weaviate_things_patch(uuid, [body])
        return
